"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Database_instances, _Database_timers, _Database_ready, _Database_timeDataTableName_get, _Database_addTimeoutTable, _Database_loadTimedData, _Database_expire;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Database = void 0;
const Table_1 = require("../structures/Table");
const DefaultDatabaseOptions_1 = require("../util/constants/DefaultDatabaseOptions");
const better_sqlite3_1 = __importDefault(require("better-sqlite3"));
const iterate_1 = __importDefault(require("../functions/iterate"));
const DatabaseError_1 = require("../structures/DatabaseError");
const DatabaseConnected_1 = __importDefault(require("../util/decorators/DatabaseConnected"));
const tiny_typed_emitter_1 = require("tiny-typed-emitter");
const __1 = require("..");
const cast_1 = __importDefault(require("../functions/cast"));
const sleep_1 = __importDefault(require("../functions/sleep"));
class Database extends tiny_typed_emitter_1.TypedEmitter {
    constructor(options = DefaultDatabaseOptions_1.DefaultDatabaseOptions) {
        super();
        _Database_instances.add(this);
        this.tables = new Map();
        _Database_timers.set(this, new Map());
        _Database_ready.set(this, false);
        this.options = options;
        __classPrivateFieldGet(this, _Database_instances, "m", _Database_addTimeoutTable).call(this);
    }
    createTable(name) {
        const table = new Table_1.Table((0, cast_1.default)(name), (0, cast_1.default)(this));
        this.tables.set(table.name, table);
        return table;
    }
    /**
     * Overwrites a timed data in the table, this will disable the other tiemout.
     * @param table The table to perform this operation on.
     * @param id The id of the primary key.
     * @param duration Duration to wait before expiring.
     * @returns
     */
    overwriteTimedData(table, id, duration, deleteOnExpire = true) {
        const timed = this.resolveTable(__classPrivateFieldGet(this, _Database_instances, "a", _Database_timeDataTableName_get));
        // This is just to verify that the table exists
        this.resolveTable(table);
        if (!timed.has([
            {
                column: "id",
                equals: id,
                and: true
            },
            {
                column: 'refTable',
                equals: table
            }
        ])) {
            return false;
        }
        const registeredAt = Date.now();
        timed.update({
            duration: {
                set: duration
            },
            registeredAt: {
                set: registeredAt
            },
            deleteOnExpire: {
                set: deleteOnExpire
            }
        }, {
            where: [
                {
                    column: "id",
                    equals: id,
                    and: true
                },
                {
                    column: 'refTable',
                    equals: table
                }
            ]
        });
        __classPrivateFieldGet(this, _Database_instances, "m", _Database_expire).call(this, table, id, duration, registeredAt, deleteOnExpire);
        return true;
    }
    /**
     * Deletes a timed data reference from the timed table, note that this will not delete the data from the actual table
     * @param table The table that was used to create the timed data on
     * @param id The primary id
     * @param duration The duration used
     * @returns
     */
    deleteTimedData(table, id) {
        const timed = this.resolveTable(__classPrivateFieldGet(this, _Database_instances, "a", _Database_timeDataTableName_get));
        // This is just to verify that the table exists
        this.resolveTable(table);
        if (!timed.has([
            {
                column: "id",
                equals: id,
                and: true
            },
            {
                column: 'refTable',
                equals: table
            }
        ])) {
            return false;
        }
        timed.delete({
            where: [
                {
                    column: 'id',
                    equals: id,
                    and: true
                },
                {
                    column: "refTable",
                    equals: table
                }
            ]
        });
        return true;
    }
    setTimedData(table, id, duration, deleteOnExpire = true) {
        // This is just to verify that the table exists
        this.resolveTable(table);
        const timed = this.resolveTable(__classPrivateFieldGet(this, _Database_instances, "a", _Database_timeDataTableName_get));
        if (timed.has([
            {
                column: 'id',
                equals: id,
                and: true
            },
            {
                column: 'refTable',
                equals: table
            }
        ])) {
            // A timed data already exists
            return false;
        }
        else {
            const registeredAt = Date.now();
            timed.insert({
                id,
                refTable: table,
                registeredAt,
                deleteOnExpire,
                duration
            });
            __classPrivateFieldGet(this, _Database_instances, "m", _Database_expire).call(this, table, id, duration, registeredAt, deleteOnExpire);
            return true;
        }
    }
    hasPrimaryKey(table) {
        return this.resolveTable(table).columns.some(c => c.data.primary);
    }
    addTable(table) {
        this.tables.set(table.name, table["setDatabase"]((0, cast_1.default)(this)));
        return table;
    }
    isReady() {
        return __classPrivateFieldGet(this, _Database_ready, "f");
    }
    getTableRawColumns(name) {
        return this.wrapper.pragma(`table_info(${name})`);
    }
    getRawTables() {
        const tables = this.wrapper.prepare(`SELECT * FROM sqlite_schema WHERE type = 'table' AND name NOT LIKE 'sqlite_%'`).all();
        for (let i = 0, len = tables.length; i < len; i++) {
            const table = tables[i];
            table.columns = this.getTableRawColumns(table.name);
        }
        return tables;
    }
    updateTables() {
        const rawTables = this.getRawTables();
        (0, iterate_1.default)(this.tables.keys(), key => {
            const table = this.tables.get(key);
            const raw = rawTables.find(c => table.name === c.name);
            if (raw) {
                for (let i = 0, len = table.columns.length; i < len; i++) {
                    const column = table.columns[i];
                    let rawColumn = raw.columns.find(c => c.name === column.name);
                    if (!rawColumn) {
                        table.createColumn(column);
                    }
                    else {
                        raw.columns = raw.columns.filter(c => c.name !== rawColumn?.name);
                        continue;
                    }
                }
                if (raw.columns.length) {
                    table["drop"]();
                }
            }
            else {
                this.wrapper.prepare(`CREATE TABLE ${table.toCreationString()}`).run();
            }
        });
    }
    resolveTable(name) {
        const table = this.tables.get((0, cast_1.default)(name));
        if (!table)
            throw new DatabaseError_1.DatabaseError('TABLE_NOT_FOUND', name);
        return table;
    }
    insert(table, data) {
        return this.resolveTable(table).insert((0, cast_1.default)(data));
    }
    getRowCount(table, ...params) {
        return this.resolveTable(table).getRowCount(...params);
    }
    insertMany(table, data) {
        return this.resolveTable(table).insertMany((0, cast_1.default)(data));
    }
    has(table, type) {
        return this.resolveTable(table).has(type);
    }
    all(table, ...params) {
        return this.resolveTable(table).all(...params);
    }
    delete(table, ...params) {
        return this.resolveTable(table).delete(...params);
    }
    update(table, data, options) {
        return this.resolveTable(table).update(data, options);
    }
    set(table, data, options) {
        return this.resolveTable(table).set(data, options);
    }
    upsert(table, data, options) {
        return this.resolveTable(table).upsert((0, cast_1.default)(data), options);
    }
    getRowPosition(table, sortOptions, whereOptions) {
        return this.resolveTable(table).getRowPosition(sortOptions, whereOptions);
    }
    get(table, type) {
        return this.resolveTable(table).get(type);
    }
    connect() {
        return new Promise(async (resolve, reject) => {
            this.wrapper = new better_sqlite3_1.default(this.options.path);
            this.updateTables();
            __classPrivateFieldSet(this, _Database_ready, true, "f");
            __classPrivateFieldGet(this, _Database_instances, "m", _Database_loadTimedData).call(this);
            this.emit("ready");
            resolve(this);
        });
    }
}
_Database_timers = new WeakMap(), _Database_ready = new WeakMap(), _Database_instances = new WeakSet(), _Database_timeDataTableName_get = function _Database_timeDataTableName_get() {
    return "__timeouts__";
}, _Database_addTimeoutTable = function _Database_addTimeoutTable() {
    if (!this.options.allowTimedData)
        return;
    process.emitWarning("DatabaseOptions#allowTimedData is a experimental option and might be bugged.");
    this.createTable(__classPrivateFieldGet(this, _Database_instances, "a", _Database_timeDataTableName_get))
        .addColumns([
        new __1.Column()
            .setName("id")
            .setType('TEXT'),
        new __1.Column()
            .setName("refTable")
            .setType('TEXT'),
        new __1.Column()
            .setName("duration")
            .setType('INT'),
        new __1.Column()
            .setName("registeredAt")
            .setType('INT'),
        new __1.Column()
            .setName("deleteOnExpire")
            .setType('BOOLEAN')
            .setDefault(false)
    ]);
}, _Database_loadTimedData = function _Database_loadTimedData() {
    if (!this.options.allowTimedData)
        return;
    const timed = this.resolveTable(__classPrivateFieldGet(this, _Database_instances, "a", _Database_timeDataTableName_get)).all();
    for (let i = 0, len = timed.length; i < len; i++) {
        const d = timed[i];
        __classPrivateFieldGet(this, _Database_instances, "m", _Database_expire).call(this, d.refTable, d.id, d.duration, d.registeredAt, d.deleteOnExpire);
    }
}, _Database_expire = function _Database_expire(table, id, duration, startedAt, deleteOnExpire) {
    const ref = this.resolveTable(table);
    const timed = this.resolveTable(__classPrivateFieldGet(this, _Database_instances, "a", _Database_timeDataTableName_get));
    // Double check that both exists
    if (!timed.has([
        {
            column: 'id',
            equals: id,
            and: true
        },
        {
            column: 'refTable',
            equals: table
        }
    ]) || !ref.has(id)) {
        return;
    }
    const fullID = `${table}_${id}`;
    // Sleep till end of timed data
    (0, sleep_1.default)(duration - (Date.now() - startedAt), 
    // Callback with the timeout
    (timer) => {
        if (__classPrivateFieldGet(this, _Database_timers, "f").has(fullID)) {
            clearTimeout(__classPrivateFieldGet(this, _Database_timers, "f").get(fullID));
            __classPrivateFieldGet(this, _Database_timers, "f").set(fullID, timer);
        }
    })
        .then(() => {
        __classPrivateFieldGet(this, _Database_timers, "f").delete(fullID);
        if (!timed.has([
            {
                column: 'id',
                equals: id,
                and: true
            },
            {
                column: 'refTable',
                equals: table
            }
        ])) {
            return;
        }
        timed.delete({
            where: [
                {
                    column: 'id',
                    equals: id,
                    and: true
                },
                {
                    column: 'refTable',
                    equals: table
                }
            ]
        });
        // If there is no reference we assume it was yeeted
        if (!ref.has(id)) {
            return;
        }
        const received = ref.get(id);
        // Delete reference
        if (deleteOnExpire) {
            ref.delete({
                where: {
                    column: 'id',
                    equals: id
                }
            });
        }
        // Emit event
        this.emit("expire", ref, received);
    });
};
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "insert", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "getRowCount", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "insertMany", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "has", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "all", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "delete", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "update", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "set", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "upsert", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "getRowPosition", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Database.prototype, "get", null);
exports.Database = Database;
//# sourceMappingURL=Database.js.map