"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createInsertQuery = exports.createUpdateQuery = exports.createSqliteQuery = void 0;
const Column_1 = require("../structures/Column");
const DatabaseError_1 = require("../structures/DatabaseError");
const wrap_1 = __importDefault(require("./wrap"));
function createSqliteQuery(options = {}) {
    const opts = [];
    if (options.where) {
        const arr = Array.isArray(options.where) ? options.where : [options.where];
        if (arr.length !== 0) {
            opts.push(`WHERE`);
        }
        for (let i = 0, len = arr.length; i < len; i++) {
            const where = arr[i];
            const isLast = i + 1 === len;
            opts.push(where.column);
            if (where.not !== undefined) {
                opts.push('NOT');
            }
            if (where.equals !== undefined) {
                opts.push('=', (0, wrap_1.default)(where.equals));
            }
            else if (where.greater !== undefined) {
                opts.push('>', (0, wrap_1.default)(where.greater));
            }
            else if (where.lesser !== undefined) {
                opts.push('<', (0, wrap_1.default)(where.lesser));
            }
            else if (where.in !== undefined) {
                opts.push(`IN`, `(${where.in.map(wrap_1.default).join(', ')})`);
            }
            else if (where.includes !== undefined) {
                opts.push('LIKE', (0, wrap_1.default)(`%${where.includes}%`));
            }
            else if (where.greaterEquals !== undefined) {
                opts.push('>=', where.greaterEquals.toString());
            }
            else if (where.lesserEquals !== undefined) {
                opts.push(`<=`, where.lesserEquals.toString());
            }
            else if (where.startsWith !== undefined) {
                opts.push(`LIKE`, (0, wrap_1.default)(`%${where.startsWith}`));
            }
            else if (where.endsWith !== undefined) {
                opts.push(`LIKE`, (0, wrap_1.default)(`${where.endsWith}%`));
            }
            else {
                throw new DatabaseError_1.DatabaseError("NO_CONDITION_FOUND", where.column);
            }
            if (!isLast) {
                opts.push(where.or ? 'OR' : 'AND');
            }
        }
    }
    if (options.sort) {
        opts.push(`ORDER BY ${Array.isArray(options.sort.columns) ? `(${options.sort.columns.join(options.sort.op ?? '+')})` : options.sort.columns} ${options.sort.order ?? 'ASC'}`);
    }
    if (options.limit !== undefined) {
        opts.push(`LIMIT ${options.limit}`);
    }
    if (options.offset !== undefined) {
        opts.push(`OFFSET ${options.offset}`);
    }
    return opts.join(' ');
}
exports.createSqliteQuery = createSqliteQuery;
function createUpdateQuery(columns, values) {
    const str = new Array();
    const len2 = columns.length;
    for (let x = 0; x < len2; x++) {
        const column = columns[x];
        const value = values[column.name];
        if (value === null || value === undefined) {
            if (value === null) {
                str.push(`${column.name} = null`);
            }
            continue;
        }
        if (value.add !== undefined) {
            str.push(`${column.name} = ${column.name} + ${value.add}`);
        }
        else if (value.set !== undefined) {
            str.push(`${column.name} = ${Column_1.Column.serialize(column, value.set)}`);
        }
        else if (value.sub !== undefined) {
            str.push(`${column.name} = ${column.name} - ${value.sub}`);
        }
        else {
            throw new DatabaseError_1.DatabaseError("NO_CONDITION_FOUND", column.name);
        }
    }
    return str.join(', ');
}
exports.createUpdateQuery = createUpdateQuery;
function createInsertQuery(columns, values) {
    if (!Array.isArray(values))
        return createInsertQuery(columns, [values]);
    const str = new Array();
    const len2 = columns.length;
    for (let i = 0, len = values.length; i < len; i++) {
        const record = values[i];
        const query = new Array();
        for (let x = 0; x < len2; x++) {
            const column = columns[x];
            const value = record[column.name] ?? null;
            const dflt = Column_1.Column.serialize(column, value);
            query.push(dflt);
        }
        if (!query.length) {
            continue;
        }
        str.push(`(${query.join(', ')})`);
    }
    return str.join(', ');
}
exports.createInsertQuery = createInsertQuery;
//# sourceMappingURL=createSqliteQuery.js.map