"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Column_instances, _Column_getStoredType;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Column = void 0;
const lodash_1 = require("lodash");
const cast_1 = __importDefault(require("../functions/cast"));
const wrap_1 = __importDefault(require("../functions/wrap"));
const ColumnDataType_1 = require("../typings/enums/ColumnDataType");
const DatabaseError_1 = require("./DatabaseError");
class Column {
    constructor(data) {
        _Column_instances.add(this);
        this.data = (0, cast_1.default)({
            value: null
        });
        this.rawDefault = null;
        if (data) {
            this.data = data;
        }
    }
    setAutoIncrementing(bool) {
        this.data.incrementing = bool;
        return this;
    }
    setDefault(value) {
        this.data.value = value;
        return this;
    }
    get adaptedDefault() {
        if (this.data.value === null)
            return null;
        if (this.rawDefault === null) {
            this.rawDefault = Column.serialize((0, cast_1.default)(this), this.data.value);
        }
        return this.rawDefault;
    }
    get default() {
        if (this.data.value !== null) {
            if (this.isJSON()) {
                return (0, lodash_1.cloneDeep)(this.data.value);
            }
            return this.data.value;
        }
        return null;
    }
    setPrimary(bool) {
        this.data.primary = bool;
        return this;
    }
    setNotNull(bool) {
        this.data.notNull = bool;
        return this;
    }
    get name() {
        return this.data.name;
    }
    toString() {
        return `${this.name} ${this.data.storedType} ${this.data.incrementing ? `AUTOINCREMENT` : ''} ${this.data.primary ? `PRIMARY KEY` : ''} ${this.data.notNull ? `NOT NULL` : ''} ${this.adaptedDefault !== null ? `DEFAULT ${this.adaptedDefault}` : ''}`;
    }
    static deserialize(column, value) {
        if (value === null || value === undefined)
            return column.default;
        switch (column.data.storedType) {
            case 'BLOB': {
                return Buffer.from(value, 'hex');
            }
            case 'TEXT': {
                if (column.isJSON()) {
                    return JSON.parse(value);
                }
                else if (column.isBigInt()) {
                    return BigInt(value);
                }
                return value;
            }
            case 'REAL': {
                return value;
            }
            case 'NUMERIC':
            case 'INTEGER':
            case 'REAL': {
                if (column.isBoolean()) {
                    return value === 1;
                }
                return value;
            }
        }
    }
    static serialize(column, value) {
        if (value === null)
            return "null";
        switch (column.data.storedType) {
            case 'BLOB': {
                if (!(value instanceof Buffer)) {
                    throw new DatabaseError_1.DatabaseError('UNEXPECTED_TYPE', value.constructor.name, 'Buffer');
                }
                value = (0, wrap_1.default)(value.toString('hex'));
                break;
            }
            case 'NUMERIC':
            case 'REAL':
            case 'INTEGER': {
                if (column.isBoolean()) {
                    value = value ? 1 : 0;
                }
                {
                    if (typeof value !== 'number') {
                        throw new DatabaseError_1.DatabaseError('UNEXPECTED_TYPE', typeof value, 'number');
                    }
                }
                break;
            }
            case 'TEXT': {
                if (column.isBigInt()) {
                    if (typeof value !== 'bigint') {
                        throw new DatabaseError_1.DatabaseError('UNEXPECTED_TYPE', typeof value, 'bigint');
                    }
                    value = (0, wrap_1.default)((0, cast_1.default)(value).toString());
                }
                else if (column.isJSON()) {
                    value = (0, wrap_1.default)(JSON.stringify(value));
                }
                else {
                    if (typeof value !== 'string') {
                        throw new DatabaseError_1.DatabaseError('UNEXPECTED_TYPE', typeof value, 'string');
                    }
                    value = (0, wrap_1.default)(value);
                }
                break;
            }
        }
        return value;
    }
    isBigInt() {
        return this.data.type === ColumnDataType_1.ColumnDataType.BIGINT;
    }
    isBoolean() {
        return this.data.type === ColumnDataType_1.ColumnDataType.BOOLEAN;
    }
    isJSON() {
        return this.data.type === ColumnDataType_1.ColumnDataType.JSON;
    }
    setType(type) {
        this.data.type = typeof type === 'string' ? ColumnDataType_1.ColumnDataType[type] : type;
        if (this.data.type === undefined) {
            throw new DatabaseError_1.DatabaseError('UNSUPPORTED_TYPE', type);
        }
        __classPrivateFieldGet(this, _Column_instances, "m", _Column_getStoredType).call(this);
        return this;
    }
    setName(name) {
        this.data.name = name;
        return this;
    }
}
exports.Column = Column;
_Column_instances = new WeakSet(), _Column_getStoredType = function _Column_getStoredType() {
    switch (this.data.type) {
        case ColumnDataType_1.ColumnDataType.INT:
        case ColumnDataType_1.ColumnDataType.INT2:
        case ColumnDataType_1.ColumnDataType.INT8:
        case ColumnDataType_1.ColumnDataType.INTEGER:
        case ColumnDataType_1.ColumnDataType.MEDIUMINT:
        case ColumnDataType_1.ColumnDataType.TINYINT: {
            this.data.storedType = 'INTEGER';
            break;
        }
        case ColumnDataType_1.ColumnDataType.BLOB: {
            this.data.storedType = 'BLOB';
            break;
        }
        case ColumnDataType_1.ColumnDataType.VARCHAR:
        case ColumnDataType_1.ColumnDataType.NVARCHAR:
        case ColumnDataType_1.ColumnDataType.NATIVE_CHARACTER:
        case ColumnDataType_1.ColumnDataType.NCHAR:
        case ColumnDataType_1.ColumnDataType.JSON:
        case ColumnDataType_1.ColumnDataType.BIGINT:
        case ColumnDataType_1.ColumnDataType.CLOB:
        case ColumnDataType_1.ColumnDataType.UNSIGNED_BIG_INT:
        case ColumnDataType_1.ColumnDataType.TEXT: {
            this.data.storedType = 'TEXT';
            break;
        }
        case ColumnDataType_1.ColumnDataType.REAL:
        case ColumnDataType_1.ColumnDataType.DOUBLE:
        case ColumnDataType_1.ColumnDataType.DOUBLE_PRECISION:
        case ColumnDataType_1.ColumnDataType.FLOAT: {
            this.data.storedType = 'REAL';
            break;
        }
        case ColumnDataType_1.ColumnDataType.NUMERIC:
        case ColumnDataType_1.ColumnDataType.DECIMAL:
        case ColumnDataType_1.ColumnDataType.BOOLEAN:
        case ColumnDataType_1.ColumnDataType.DATE:
        case ColumnDataType_1.ColumnDataType.DATETIME: {
            this.data.storedType = 'NUMERIC';
            break;
        }
    }
};
//# sourceMappingURL=Column.js.map