"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Table_db;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Table = void 0;
const createSqliteQuery_1 = require("../functions/createSqliteQuery");
const insertToUpdate_1 = __importDefault(require("../functions/insertToUpdate"));
const sanitize_1 = __importDefault(require("../functions/sanitize"));
const updateToInsert_1 = __importDefault(require("../functions/updateToInsert"));
const DatabaseConnected_1 = __importDefault(require("../util/decorators/DatabaseConnected"));
const DatabaseError_1 = require("./DatabaseError");
class Table {
    constructor(name, db) {
        this.columns = new Array();
        _Table_db.set(this, void 0);
        this.name = name;
        if (db) {
            __classPrivateFieldSet(this, _Table_db, db, "f");
        }
    }
    toValuesString() {
        return `(${this.columns.map(c => c.data.name).join(', ')})`;
    }
    setDatabase(db) {
        __classPrivateFieldSet(this, _Table_db, db, "f");
        return this;
    }
    get(type) {
        if (typeof type === 'string') {
            const column = this.primary;
            if (!column) {
                throw new DatabaseError_1.DatabaseError("NO_PRIMARY_COLUMN");
            }
            return this.get({
                column: column.name,
                equals: type
            });
        }
        else {
            const query = (0, createSqliteQuery_1.createSqliteQuery)({
                where: type
            });
            const full = `SELECT * FROM ${this.name} ${query}`;
            const data = __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(full)
                .get();
            const t = (0, sanitize_1.default)(this.columns, data)[0];
            this.emit('get', this, {
                where: type,
                query: full
            }, t);
            return t;
        }
    }
    delete(options) {
        const query = `DELETE FROM ${this.name} ${(0, createSqliteQuery_1.createSqliteQuery)(options)}`;
        const res = this.db.wrapper.prepare(query).run();
        this.emit('delete', this, {
            ...options,
            query
        }, res);
        return res;
    }
    emit(event, ...args) {
        __classPrivateFieldGet(this, _Table_db, "f").emit(event, ...args);
        const query = args[1]?.query;
        if (query) {
            __classPrivateFieldGet(this, _Table_db, "f").emit("query", this, query);
        }
        return this;
    }
    getRowPosition(sortOptions, whereOptions) {
        const sort = (0, createSqliteQuery_1.createSqliteQuery)({ sort: sortOptions });
        const opts = Array.isArray(whereOptions) ? whereOptions : [whereOptions];
        const where = (0, createSqliteQuery_1.createSqliteQuery)({ where: whereOptions });
        const data = __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(`
        SELECT * FROM ( 
            SELECT
            ROW_NUMBER () OVER ( 
            ${sort} 
            ) 
                RowNum,
                ${opts.map(c => c.column).join(', ')}
            FROM
                ${this.name}
            )
        ${where}
        `).all();
        return data[0]?.RowNum ?? null;
    }
    get db() {
        return __classPrivateFieldGet(this, _Table_db, "f");
    }
    has(type) {
        if (typeof type === 'string') {
            const column = this.primary;
            if (!column) {
                throw new DatabaseError_1.DatabaseError("NO_PRIMARY_COLUMN");
            }
            return this.has({
                column: column.name,
                equals: type
            });
        }
        else {
            const arr = Array.isArray(type) ? type : [type];
            const query = (0, createSqliteQuery_1.createSqliteQuery)({
                where: type
            });
            const full = `SELECT ${arr.map(c => c.column).join(', ')} FROM ${this.name} ${query}`;
            const data = __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(full)
                .get();
            const bool = Boolean(data);
            this.emit("has", this, {
                where: type,
                query: full
            }, bool);
            return bool;
        }
    }
    getRowCount(options = {}) {
        const query = (0, createSqliteQuery_1.createSqliteQuery)(options);
        const got = __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(`SELECT COUNT(*) as count FROM ${this.name} ${[query]}`).all()[0];
        return got.count;
    }
    drop() {
        __classPrivateFieldGet(this, _Table_db, "f").wrapper.pragma(`foreign_keys=off`);
        __classPrivateFieldGet(this, _Table_db, "f").wrapper.transaction(() => {
            __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(`ALTER TABLE ${this.name} RENAME TO _${this.name}_old`).run();
            __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(`CREATE TABLE ${this.toCreationString()}`).run();
            __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(`
                INSERT INTO ${this.name}${this.toValuesString()}
                SELECT ${this.columns.map(c => c.name).join(', ')}
                FROM _${this.name}_old
                `);
            __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(`DROP TABLE _${this.name}_old`).run();
        })();
    }
    set(data, options) {
        if (this.has(options)) {
            this.update(data, { where: options });
            return true;
        }
        else {
            const d = (0, updateToInsert_1.default)(this.columns, data);
            this.insert(d);
            return false;
        }
    }
    update(data, options) {
        const query = (0, createSqliteQuery_1.createUpdateQuery)(this.columns, data);
        const opts = (0, createSqliteQuery_1.createSqliteQuery)(options);
        const full = `UPDATE ${this.name} SET ${query} ${opts}`;
        const ran = __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(full).run();
        this.emit("update", this, {
            ...options,
            query: full
        }, data, ran);
        return ran;
    }
    all(options) {
        const query = (0, createSqliteQuery_1.createSqliteQuery)(options);
        const full = `SELECT * FROM ${this.name} ${query}`;
        const all = __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(full).all();
        const san = (0, sanitize_1.default)(this.columns, all, __classPrivateFieldGet(this, _Table_db, "f").options.length ?? 0);
        this.emit("all", this, {
            ...(options ?? {}),
            query: full
        }, san);
        return san;
    }
    insertMany(data) {
        const query = (0, createSqliteQuery_1.createInsertQuery)(this.columns, data);
        const full = `INSERT INTO ${this.name}${this.toValuesString()} VALUES ${query}`;
        const ran = __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(full).run();
        this.emit("insert", this, {
            query: full
        }, data, ran);
        return ran;
    }
    upsert(data, options) {
        if (this.has(options)) {
            const d = (0, insertToUpdate_1.default)(this.columns, data);
            this.update(d, { where: options });
            return true;
        }
        else {
            this.insert(data);
            return false;
        }
    }
    insert(data) {
        const query = (0, createSqliteQuery_1.createInsertQuery)(this.columns, data);
        const full = `INSERT INTO ${this.name}${this.toValuesString()} VALUES ${query}`;
        const ran = __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(full).run();
        this.emit("insert", this, {
            query: full
        }, data, ran);
        return ran;
    }
    toCreationString() {
        return `${this.name}(${this.columns.map(c => c.toString()).join(', ')})`;
    }
    createColumn(column) {
        __classPrivateFieldGet(this, _Table_db, "f").wrapper.prepare(`ALTER TABLE ${this.name} ADD COLUMN ${column.toString()}`).run();
        return this;
    }
    hasColumn(column) {
        return this.columns.some(c => c.name === column.name);
    }
    get primary() {
        return this.columns.find(c => c.data.primary) ?? null;
    }
    addColumns(...columns) {
        this.columns.push(...columns.flat(1));
        return this;
    }
}
_Table_db = new WeakMap();
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "get", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "delete", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "getRowPosition", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "has", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "getRowCount", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "set", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "update", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "all", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "insertMany", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "upsert", null);
__decorate([
    (0, DatabaseConnected_1.default)()
], Table.prototype, "insert", null);
exports.Table = Table;
//# sourceMappingURL=Table.js.map